//  Pendulum Control
//  Ansi-C written for Microchip's PIC-X compiler.

#include <xc.h>
#include <pic12f683.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdbool.h>

// INSTRUCTIONS TO COMPILER
#define _XTAL_FREQ                  8000000
#define  test_bit(var, bit)   ((var) & (1 <<(bit)))
#define  set_bit(var, bit)    ((var) |= (1 << (bit)))
#define  clear_bit(var, bit)    ((var) &= ~(1 << (bit)))


// CONFIGURATION BYTES

// CONSTANTS
#define false						0
#define true						1
#define pulse_dur_2400bps           400   //in microseconds. Tuned to produce the right pulse width. Must produce  a 417us pulse
#define pulse_dur_9600bps           90   //in microseconds. Tuned to produce the right pulse width. Must produce  a 104us pulse
#define serial_tx_pin                   1


// GLOBAL VARIABLES
unsigned char i; //Used as counter
unsigned char j; //Used as counter


//******************************************************************************************************


// Timer routines
void delay_ms(int delay)
{
    for (i = 0; i < delay; i++) __delay_ms(1);
}
void delay_100us(int delay)
{
    for (i = 0; i < delay; i++) __delay_us(100);
}

// Software Tx-Only USART, sends one character.
//The code for the TX part of the USART is simplified in that it generates 10 bits of serial data with no parity bit.
//The basic PIC Serial port configuration for the TX software is:

//Bits per second (BAUD) 	2400
//Number of bits 	8
//Parity 	None
//Stop bits 	1
//Flow control 	None

//Since each bit takes 1/2400 seconds the total time to transmit a digit is 4.16ms.  
//Note: The bit timing won't be exact but it will be good enough for bench top use.

void putch(unsigned char tx_char) // must be called "putch" in order to work with the printf command
// This is a software tx uart
{
    unsigned short mask = 1;
    unsigned char i;
    unsigned short txdata;

    txdata = tx_char << 1; // shift left by one the ASCII character to send. Left with 7 bits.
    // start bit (0) + 7 data bits + bit (0)
    for (i = 0; i < 9; i++) // lsb first
    {
        if (mask & txdata) set_bit(GPIO, serial_tx_pin); // Transfer data to the output pin
        else clear_bit(GPIO, serial_tx_pin);             //    "
        mask <<= 1;                          // Shift mask left by 1
        __delay_us(pulse_dur_2400bps); // wait one bit width duration
    }
    set_bit(GPIO, serial_tx_pin); // add stop bit (1)
    __delay_us(pulse_dur_2400bps); // wait one bit width duration for stop bit
    __delay_ms(1); // Pause between characters
}

void main()
{
    //Set CPU to 8MHz Here.
    // Initialization stage
    INTCON      = 0b00000000; // disable interrupts
    IOC         = 0b00000000; // disable GPIO interrupts on change
    ...
    ...
    TRISIO      = 0b00111100; // GPIO first disabled, but assigned as follows:
    //	5-input, 
    //	4-input, Serial Data Rx
    //	3-Pin is input only, no weak pullup.
    //	2-input, 
    //	1-output, Serial data Tx
    //	0-output, 

    //Main loop
    while (true)
    {
	...
	...
	...
        // Now formatting and sending the serial string
        __delay_ms(100);    // Pause for 100 milliseconds
        printf("%+5.5d,%3.3d,%3.3d,", t1_ccp_diff,coil_pulse_duration_ms,coil_pulse_duration_us); // Send formatted data string to the serial output pin.
        printf("%c",10); // Send a Line Feed character
    }
}
